//first included your library
#include <OneWire.h>
#include <DallasTemperature.h>
#include <SoftwareSerial.h>
#include <Wire.h>
#include <dht.h>
#include <EEPROM.h>
#include <avr/pgmspace.h>

//SIM900 settings
#define PIN_POWER    6
#define PIN_RESET    7
#define PIN_TX    3
#define PIN_RX    2
#define SS_BAUDRATE  9600
#define UART_BUFFER 350
#define PIN_BUFFER 8
#define MESSAGE_TIMEOUT 2
#define SYMBOL_TIMEOUT 30
#define NO_TIMEOUT 0
#define EndMark (char)26
#define PHONE_BUFFER 15
#define EMAIL_BUFFER 30
#define SMTP_BUFFER 20
#define PORT_BUFFER 8
#define PWD_BUFFER 12

SoftwareSerial SIM900(PIN_TX, PIN_RX); 
int SimEnable=0;
int csq=99;
int scoreHRN=0;
int scoreKOP=0;
char uartbuf[UART_BUFFER];
char pinbuf[PIN_BUFFER];
char pincode[PIN_BUFFER];
char phone[PHONE_BUFFER];
char email[EMAIL_BUFFER];
char email_login[EMAIL_BUFFER];
char email_smtp[SMTP_BUFFER];
char email_port[PORT_BUFFER];
char email_pwd[PWD_BUFFER];
int pinOk=0;
int pinCount=0;
int menuCount=0;
int ringActive=0;
int alerts[5]={0,0,0,0,0};

//i2c settings
#define I2C_MSG_BUFF 186
#define BUFF_SIZE 6*10+I2C_MSG_BUFF+6
#define SLAVE_ADDRESS 0x04
int number = 0;
int state = 0;
int i2cActive=0;
int i2cActiveTime=0;
char i2c_buff[BUFF_SIZE]={
//SENSOR DATA
0x00,0x00,0x00,0x00,0x00,0x00,//GAS
0x00,0x00,0x00,0x00,0x00,0x00,//WATER
0x00,0x00,0x09,0x00,0x09,0x00,//ELECTRICITY
0x00,0x00,0x00,0x00,0x00,0x00,//SIGNALIZATION
0x00,0x00,0x00,0x00,0x00,0x00,//INTERNET
0x00,0x00,0x00,0x00,0x00,0x00,//TEMPERATURE
0x00,0x00,0x00,0x00,0x00,0x00,//HUMIDITY
0x00,0x00,0x00,0x00,0x00,0x00,//BRIGHTNESS
0x00,0x00,0x00,0x00,0x00,0x00,//CO2
0x00,0x00,0x00,0x00,0x00,0x00, //SIM900

//MESSAGE BUFFER
0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,0x00,0x00,

0x00,0x00,  //Function request
0x00,0x00,  //Interupt request
0x00,0x00  
};

const char name_0[] PROGMEM = "GAS";   // "String 0" etc are strings to store - change to suit.
const char name_1[] PROGMEM = "WATER";
const char name_2[] PROGMEM = "ELECTR";
const char name_3[] PROGMEM = "SIGNAL";
const char name_4[] PROGMEM = "INET";
const char name_5[] PROGMEM = "TEMP";
const char name_6[] PROGMEM = "HUM";
const char name_7[] PROGMEM = "BRIGHT";
const char name_8[] PROGMEM = "CO2";
const char name_9[] PROGMEM = "SIM900";

const char* const names[] PROGMEM = {name_0, name_1, name_2, name_3, name_4, name_5, name_6, name_7, name_8, name_9};

const char* names_short[] PROGMEM ={"G","W","E","S","I","T","H","B","C","S"};

const char alert_0[] PROGMEM = "ALERT";   // "String 0" etc are strings to store - change to suit.
const char alert_1[] PROGMEM = "ALERT: GAS LEAK";
const char alert_2[] PROGMEM = "ALERT: WATER LEAK";
const char alert_3[] PROGMEM = "ALERT: ELECTRICITY LOST";
const char alert_4[] PROGMEM = "ALERT: ALARM TRIGGERED";
const char alert_5[] PROGMEM = "ALERT: INTERNET LOST";

const char* const alert_names[] PROGMEM = {alert_0, alert_1, alert_2, alert_3, alert_4, alert_5};

//DHT11 settings
#define DHT_BUS 15
DHT DHT11 = DHT();

//18b20 Settings
#define ONE_WIRE_BUS 14 // Data wire is plugged into port 14 on the Arduino
OneWire  ds(ONE_WIRE_BUS);  
DallasTemperature DS18B20(&ds); // Pass our oneWire reference to Dallas Temperature. 
DeviceAddress insideThermometer; // arrays to hold device address

//Functions
void(* resetFunc) (void) = 0;

int freeRam () 
{
  extern int __heap_start, *__brkval; 
  int v; 
  return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval); 
}

const unsigned char Crc8Table[256] = {
  0x00, 0x5E, 0xBC, 0xE2, 0x61, 0x3F, 0xDD, 0x83,
  0xC2, 0x9C, 0x7E, 0x20, 0xA3, 0xFD, 0x1F, 0x41,
  0x9D, 0xC3, 0x21, 0x7F, 0xFC, 0xA2, 0x40, 0x1E,
  0x5F, 0x01, 0xE3, 0xBD, 0x3E, 0x60, 0x82, 0xDC,
  0x23, 0x7D, 0x9F, 0xC1, 0x42, 0x1C, 0xFE, 0xA0,
  0xE1, 0xBF, 0x5D, 0x03, 0x80, 0xDE, 0x3C, 0x62,
  0xBE, 0xE0, 0x02, 0x5C, 0xDF, 0x81, 0x63, 0x3D,
  0x7C, 0x22, 0xC0, 0x9E, 0x1D, 0x43, 0xA1, 0xFF,
  0x46, 0x18, 0xFA, 0xA4, 0x27, 0x79, 0x9B, 0xC5,
  0x84, 0xDA, 0x38, 0x66, 0xE5, 0xBB, 0x59, 0x07,
  0xDB, 0x85, 0x67, 0x39, 0xBA, 0xE4, 0x06, 0x58,
  0x19, 0x47, 0xA5, 0xFB, 0x78, 0x26, 0xC4, 0x9A,
  0x65, 0x3B, 0xD9, 0x87, 0x04, 0x5A, 0xB8, 0xE6,
  0xA7, 0xF9, 0x1B, 0x45, 0xC6, 0x98, 0x7A, 0x24,
  0xF8, 0xA6, 0x44, 0x1A, 0x99, 0xC7, 0x25, 0x7B,
  0x3A, 0x64, 0x86, 0xD8, 0x5B, 0x05, 0xE7, 0xB9,
  0x8C, 0xD2, 0x30, 0x6E, 0xED, 0xB3, 0x51, 0x0F,
  0x4E, 0x10, 0xF2, 0xAC, 0x2F, 0x71, 0x93, 0xCD,
  0x11, 0x4F, 0xAD, 0xF3, 0x70, 0x2E, 0xCC, 0x92,
  0xD3, 0x8D, 0x6F, 0x31, 0xB2, 0xEC, 0x0E, 0x50,
  0xAF, 0xF1, 0x13, 0x4D, 0xCE, 0x90, 0x72, 0x2C,
  0x6D, 0x33, 0xD1, 0x8F, 0x0C, 0x52, 0xB0, 0xEE,
  0x32, 0x6C, 0x8E, 0xD0, 0x53, 0x0D, 0xEF, 0xB1,
  0xF0, 0xAE, 0x4C, 0x12, 0x91, 0xCF, 0x2D, 0x73,
  0xCA, 0x94, 0x76, 0x28, 0xAB, 0xF5, 0x17, 0x49,
  0x08, 0x56, 0xB4, 0xEA, 0x69, 0x37, 0xD5, 0x8B,
  0x57, 0x09, 0xEB, 0xB5, 0x36, 0x68, 0x8A, 0xD4,
  0x95, 0xCB, 0x29, 0x77, 0xF4, 0xAA, 0x48, 0x16,
  0xE9, 0xB7, 0x55, 0x0B, 0x88, 0xD6, 0x34, 0x6A,
  0x2B, 0x75, 0x97, 0xC9, 0x4A, 0x14, 0xF6, 0xA8,
  0x74, 0x2A, 0xC8, 0x96, 0x15, 0x4B, 0xA9, 0xF7,
  0xB6, 0xE8, 0x0A, 0x54, 0xD7, 0x89, 0x6B, 0x35
};

unsigned char Crc8(unsigned char *pcBlock, unsigned char len)
{
  unsigned char crc = 0;
  int i=0;
  for (i = 0; i < len; i++)
    crc = Crc8Table[crc ^ *pcBlock++];
  return crc;
}

void PIN_setup()
{
  pinMode(A2, INPUT_PULLUP); 
  pinMode(A3, INPUT_PULLUP); 
  pinMode(A6, INPUT_PULLUP); 
  pinMode(A7, INPUT_PULLUP); 
}

// function to print a device address
void printAddress(DeviceAddress deviceAddress)
{
  for (uint8_t i = 0; i < 8; i++)
  {
    if (deviceAddress[i] < 16) Serial.print("0");
    Serial.print(deviceAddress[i], HEX);
  }
}

void D18B20_setup()
{
  DS18B20.begin();
  if (!DS18B20.getAddress(insideThermometer, 0)) Serial.println(F("...Error")); 
  else Serial.println(F("... Ok")); 
  //printAddress(insideThermometer);
  //DS18B20.setResolution(insideThermometer, 9);
}

void setup()
{
  Serial.begin(9600);
  Serial.println(F("Start setup ... ")); 

  //Setup 18b20
  Serial.print(F("Setup DS18B20 ")); 
  D18B20_setup();

  //Setup dht11
  Serial.print(F("Setup DHT")); 
  DHT11.attach(DHT_BUS);
  DHT11.update();
  if (DHT11.getLastError()==DHT_ERROR_OK)Serial.println(F("... Ok")); 
  else Serial.println(F("... Error")); 

  //Setup ADC
  Serial.print(F("Setup ADC")); 
  PIN_setup();
  Serial.println(F("... Ok")); 

  //Setup I2C
  Serial.print(F("Setup I2C")); 
  Wire.begin(SLAVE_ADDRESS);  // initialize i2c as slave
  Wire.onReceive(I2C_RecData); // define callbacks for i2c communication
  Wire.onRequest(I2C_SendData);
  Serial.println(F("... Ok")); 

  //Setup phone number
  for(int i=0; i<15; i++)phone[i]=EEPROM.read(i);
  Serial.print(F("Setup emergency phone ... phone: ")); 
  Serial.println(phone);
  
  //Setup mail 
  for(int i=0; i<30; i++)email[i]=EEPROM.read(i+15);
  Serial.print(F("Setup emergency email ... email: ")); 
  Serial.println(email);
  
  //Setup mail 
  for(int i=0; i<EMAIL_BUFFER; i++)email_login[i]=EEPROM.read(i+PHONE_BUFFER+EMAIL_BUFFER);
  for(int i=0; i<SMTP_BUFFER; i++)email_smtp[i]=EEPROM.read(i+PHONE_BUFFER+EMAIL_BUFFER+EMAIL_BUFFER);
  for(int i=0; i<PORT_BUFFER; i++)email_port[i]=EEPROM.read(i+PHONE_BUFFER+EMAIL_BUFFER+EMAIL_BUFFER+SMTP_BUFFER);
  for(int i=0; i<PWD_BUFFER; i++)email_pwd[i]=EEPROM.read(i+PHONE_BUFFER+EMAIL_BUFFER+EMAIL_BUFFER+SMTP_BUFFER+PORT_BUFFER);
  Serial.print(F("Setup account email ... email: ")); 
  Serial.println(email_login);
  
  //Setup SIM9000
  Serial.print(F("Setup SIM900")); 
  for(int i=0; i<PIN_BUFFER; i++)pincode[i]=EEPROM.read(i+PHONE_BUFFER+EMAIL_BUFFER+EMAIL_BUFFER+SMTP_BUFFER+PORT_BUFFER+PWD_BUFFER);
  SIM900_pin_config();
  SIM900.begin(SS_BAUDRATE);
  SIM900.print(F("AT\r"));
  if(SIM900_waitOK())
  {
    Serial.println(F("... Ok")); 
    SIM900_setup();    
  }
  else
  {
    Serial.println("");
    SIM900_power();
    SIM900.print(F("AT\r"));
    if(SIM900_waitOK())
    {
      Serial.println(F("... Ok"));
      SIM900_setup();   
      goto endsetup;
    } 
    else Serial.println(F("SIM900 ... Error"));   

    SIM900_reset();
    SIM900_power();
    SIM900.print(F("AT\r"));
    if(SIM900_waitOK())
    {
      Serial.println(F("SIM900 ... Ok")); 
      SIM900_setup();   
    }
    else Serial.println(F("SIM900 ... Error"));   
  }
endsetup:

  Serial.print(F("Setup complete\r\nFree RAM: ")); 
  Serial.println(freeRam()); 
}

void loop()
{
  //18b20
  DS18B20.requestTemperatures(); // Send the command to get temperatures
  float tempC = DS18B20.getTempC(insideThermometer);
  int cels=0,celsm=0;
  if (!DS18B20.getAddress(insideThermometer, 0)) Serial.println(F("Error: 18b20 lost")); 
  else 
  {
    cels=(int)tempC;
    celsm=(tempC-cels)*100;

    i2c_buff[30]=cels;
    i2c_buff[31]=celsm;
  }

  //MQ-2
  int ppm=0;
  float sensor_volt,RS_gas,ratio;
  float sensorValue=0;
  for(int x = 0 ; x < 100 ; x++)sensorValue = sensorValue + analogRead(A3);
  sensorValue = sensorValue/100.0;
  sensor_volt=(float)sensorValue/1024*5.0;
  RS_gas = (5.0-sensor_volt)/sensor_volt; 
  ratio = 11-RS_gas/2.52;  
  if(ratio<5.5)ppm=ratio*20;
  else if(ratio>=5.5 && ratio<6.5)ppm=ratio*30;
  else if(ratio>=6.5 && ratio<6.7)ppm=ratio*44;
  else if(ratio>=6.7 && ratio<6.9)ppm=ratio*60;
  else if(ratio>=6.9 && ratio<7.1)ppm=ratio*72;
  else if(ratio>=7.1 && ratio<7.3)ppm=ratio*84;
  else if(ratio>=7.3 && ratio<7.5)ppm=ratio*95;
  else if(ratio>=7.5 && ratio<7.7)ppm=ratio*105;
  else if(ratio>=7.7 && ratio<7.9)ppm=ratio*120;
  else if(ratio>=7.9)ppm=ratio*150;

  if(ppm<0)ppm=0;
  
  i2c_buff[0]=ppm/100;
  i2c_buff[1]=ppm%100;

  //Brightness control
  float brt=0;
  for(int x = 0 ; x < 100 ; x++)brt = brt + analogRead(A7);
  brt = brt/100.0;
  i2c_buff[42]=((int)brt)/10;
  i2c_buff[43]=((int)brt)%10;

  //Light control
  float light=0;
  for(int x = 0 ; x < 100 ; x++)light = light + analogRead(A6);
  light = light/100.0;
  i2c_buff[44]=((int)light)/10;
  i2c_buff[45]=((int)light)%10;

  //Voltage control
  float voltage=0;
  for(int x = 0 ; x < 100 ; x++)voltage = voltage + analogRead(A2);
  voltage = voltage/100.0;
  voltage = 1243*voltage/1024;
  i2c_buff[12]=((int)voltage)/100;
  i2c_buff[13]=((int)voltage)%100;

  //dht11
  DHT11.update();
  switch (DHT11.getLastError())
  {
  case DHT_ERROR_OK:
    i2c_buff[32]=DHT11.getTemperatureInt();
    i2c_buff[36]=DHT11.getHumidityInt();
    break;
  case DHT_ERROR_START_FAILED_1:
    Serial.println(F("Error: DHT11 start failed (stage 1)"));
    break;
  case DHT_ERROR_START_FAILED_2:
    Serial.println(F("Error: DHT11 start failed (stage 2)"));
    break;
  case DHT_ERROR_READ_TIMEOUT:
    Serial.println(F("Error: DHT11 read timeout"));
    break;
  case DHT_ERROR_CHECKSUM_FAILURE:
    Serial.println(F("Error: DHT11 checksum error"));
    break;
  }

  //SIM900
  i2c_buff[54]=scoreHRN;
  i2c_buff[55]=scoreKOP;
  
  i2cActive=i2c_buff[6*10+I2C_MSG_BUFF]+i2c_buff[6*10+I2C_MSG_BUFF+2]+i2cActiveTime;

  if(!ringActive && !i2cActive)
  {
    SIM900.print(F("AT+CSQ\r"));
    if(SIM900_wait("+CSQ:",MESSAGE_TIMEOUT,SYMBOL_TIMEOUT,uartbuf))
    {
      String currStr=String(uartbuf);
      csq=(currStr.substring(currStr.indexOf(" ")+1,currStr.indexOf(","))).toInt();
      i2c_buff[56]=csq;
    }
    else 
    {
      i2c_buff[56]=0;
      Serial.println(F("Error: SIM900 read timeout"));
    }
    
    SIM900.print(F("AT+CBC\r"));
    if(SIM900_wait("+CBC:",MESSAGE_TIMEOUT,SYMBOL_TIMEOUT,uartbuf))
    {
      char *s;
      s= strstr(uartbuf,",")+1;
      String currStr=String(s);
      i2c_buff[58]=(currStr.substring(0,currStr.indexOf(","))).toInt();
    }
    else 
    {
      i2c_buff[58]=0;
      Serial.println(F("Error: SIM900 read timeout"));
    }
  }
  
  for(int i=0; i<5000;i++)
  {
    if(i2cActiveTime>0)i2cActiveTime--;
    
    if(SIM900_listen())
    {

      if(NULL != strstr(uartbuf,"RING")) 
      {
        i2c_buff[6*10+I2C_MSG_BUFF+2]=3;
        pinOk=0;
        pinCount=0;
        ringActive=1;
        memset(pinbuf,0,PIN_BUFFER);
        SIM900_change_mic_gain("0","0");
        SIM900_answer();
        SIM900_amr_play("001.amr");
      }
      else if((NULL != strstr(uartbuf,"NO CARRIER"))||(NULL != strstr(uartbuf,"NO DIALTONE"))||(NULL != strstr(uartbuf,"BUSY"))||(NULL != strstr(uartbuf,"NO ANSWER")))
      {
        ringActive=0;
        pinOk=0;
        pinCount=0;
        memset(pinbuf,0,PIN_BUFFER);
        SIM900_change_mic_gain("0","0");
      }
      else if(NULL != strstr(uartbuf,"AMR_STOP"))
      {
        if(menuCount>0)
        {
          int a=0;
          switch(menuCount)
          {
            case 1:
              SIM900_amr_play("007.amr");
              menuCount=2;
            break;
            
            case 2:
              for(int i=0; i<6; i+=2)
              {
                if(i2c_buff[i]>0)a++;
              }
              if(a>0)SIM900_amr_play("006.amr");
              else SIM900_amr_play("005.amr");
              menuCount=3;
            break;
            
            case 3:
              SIM900_amr_play("008.amr");
              menuCount=4;
            break;
            
            case 4:
              for(int i=0; i<6; i+=2)
              {
                if(i2c_buff[i+6]>1)a++;
              }
              if(a>0)SIM900_amr_play("006.amr");
              else SIM900_amr_play("005.amr");
              menuCount=5;
            break;
            
            case 5:
              SIM900_amr_play("009.amr");
              menuCount=6;
            break;
            
            case 6:
              for(int i=0; i<6; i+=2)
              {
                if(i2c_buff[i+12]<1)a++;
              }
              if(a>0)SIM900_amr_play("006.amr");
              else SIM900_amr_play("005.amr");
              menuCount=7;
            break;
            
            case 7:
              SIM900_amr_play("010.amr");
              menuCount=8;
            break;
            
            case 8:
              for(int i=0; i<6; i+=2)
              {
                if(i2c_buff[i+24]>4)a++;
              }
              if(a>0)SIM900_amr_play("006.amr");
              else SIM900_amr_play("005.amr");
              menuCount=9;
            break;
            
            case 9:
              SIM900_amr_play("011.amr");
              menuCount=10;
            break;
            
            case 10:
              for(int i=0; i<6; i+=2)
              {
                if(i2c_buff[i+18]>1)a++;
              }
              if(a>0)SIM900_amr_play("006.amr");
              else SIM900_amr_play("005.amr");
              menuCount=0;
            break;
            
            default:
            break;
          }
        }
      }
      else if(NULL != strstr(uartbuf,"+CMTI: \"SM\"")) 
      { 
        char *s;
        s = strstr(uartbuf,"+CMTI: \"SM\"");
        i2c_buff[6*10+I2C_MSG_BUFF+2]=2;
        i2c_buff[6*10+I2C_MSG_BUFF+3]=atoi(s+12);
      }
      else if(NULL != strstr(uartbuf,"+DTMF:"))
      {
        char *s;
        s=strstr(uartbuf,"+DTMF:");
        int num = (s-uartbuf);
        if(!pinOk)
        {
          switch(uartbuf[num+6])
          {
          case '*':
           Serial.print(F("PIN: "));
           Serial.println(pinbuf);
            if(NULL != strstr(pinbuf,pincode))
            {
              pinOk=1;
              SIM900_amr_stop();
              SIM900_amr_play("002.amr");
            }
            else
            {              
              pinOk=0;
              pinCount=0;
              memset(pinbuf,0,PIN_BUFFER);
              SIM900_hangup();
            }
            break;

          case '#':
            pinOk=0;
            pinCount=0;
            memset(pinbuf,0,PIN_BUFFER);
            break;

          default:
            pinbuf[pinCount]=uartbuf[num+6];
            if(pinCount<7)pinCount++;
            break;
          }
        }
        else
        {
          switch(uartbuf[num+6])
          {
          case '1':
            menuCount=0;
            SIM900_change_mic_gain("0","15");
            SIM900_amr_stop();
            SIM900_amr_play("003.amr");
            break;

          case '5': //Voice
            menuCount=1;
            SIM900_amr_play("004.amr");
            break;
            
          case '3': //EMail
            //SIM900_amr_play("004.amr");
            SIM900_hangup();
            delay(2000);
            strcpy(uartbuf,"SENSORS\r\n");
            for(int j=0;j<10;j++)
            {
              for(int i=0;i<6;i+=2)
              {
                char msg[5];
                itoa(i2c_buff[i+j*6],msg,10);
                strcat(uartbuf,msg);
                strcat(uartbuf,".");
                itoa(i2c_buff[i+j*6+1],msg,10);
                strcat(uartbuf,msg);
                strcat(uartbuf," ");
              }
              strcat(uartbuf,"- ");
              strcat_P(uartbuf, (char*)pgm_read_word(&(names[j])));
              //strcat(uartbuf, names[j]);
              strcat(uartbuf,"\r\n");
            }
            SIM900_sendMail(email, "Sensors values", uartbuf);
            break;
            
          case '7': //SMS
            SIM900_hangup();
            memset(uartbuf,0,UART_BUFFER);
            delay(2000);
            for(int j=0;j<5;j++)
            {
              for(int i=0;i<6;i+=2)
              {
                char msg[5];
                itoa(i2c_buff[i+j*6],msg,10);
                strcat(uartbuf,msg);
                strcat(uartbuf,".");
                itoa(i2c_buff[i+j*6+1],msg,10);
                strcat(uartbuf,msg);
                strcat(uartbuf," ");
              }
              strcat(uartbuf,"-");
              char c[1]={pgm_read_byte_near(names_short + j)};
              strcat(uartbuf, c);
              strcat(uartbuf,"\r\n");
            }
            SIM900_send_sms(phone,uartbuf);
            break;
            
          case '4': //SMS
            SIM900_hangup();
            memset(uartbuf,0,UART_BUFFER);            
            delay(2000);
            for(int j=5;j<10;j++)
            {
              for(int i=0;i<6;i+=2)
              {
                char msg[5];
                itoa(i2c_buff[i+j*6],msg,10);
                strcat(uartbuf,msg);
                strcat(uartbuf,".");
                itoa(i2c_buff[i+j*6+1],msg,10);
                strcat(uartbuf,msg);
                strcat(uartbuf," ");
              }
              strcat(uartbuf,"-");
              char c[1]={pgm_read_byte_near(names_short + j)};
              strcat(uartbuf, c);
              strcat(uartbuf,"\r\n");
            }
            SIM900_send_sms(phone,uartbuf);
            break;
            
          case 0:
            for(int x = 0 ; x < 5 ; x++)alerts[x]=0;
            break;

          default:
            menuCount=0;
            break;
          }
        }
      }
      else
      {
          char *p;
          p=i2c_buff+60;
          memcpy(p,uartbuf,I2C_MSG_BUFF);
          i2c_buff[6*10+I2C_MSG_BUFF+2]=1;
          i2cActiveTime=0;
      }
      memset(uartbuf,0,UART_BUFFER);
    }
    
    char *s;
    s=i2c_buff+60;
    
    switch(i2c_buff[6*10+I2C_MSG_BUFF])
    {
      //System functions
      
      case 1:
        resetFunc();
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        break;
        
      case 2:
        SIM900_reset();
        SIM900_power();
        SIM900.print(F("AT\r"));
        if(SIM900_waitOK())
        {
          Serial.println(F("SIM900 ... Ok")); 
          SIM900_setup();   
        }
        else Serial.println(F("SIM900 ... Error"));  
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        break; 
        
      case 3:
        I2C_ClearMsgBuf();
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        break;
      
      //Configure parameters
      
      case 11:
        I2C_SetPhone_Temp(i2c_buff[6*10+I2C_MSG_BUFF+1]);
        I2C_ClearMsgBuf();
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        break;
  
      case 12:
        I2C_SetEMail_Temp(i2c_buff[6*10+I2C_MSG_BUFF+1]);
        I2C_ClearMsgBuf();
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        break;  
      
      case 13:
        I2C_SetPhone(i2c_buff[6*10+I2C_MSG_BUFF+1]);
        I2C_ClearMsgBuf();
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        break;
  
      case 14:
        I2C_SetEMail(i2c_buff[6*10+I2C_MSG_BUFF+1]);
        I2C_ClearMsgBuf();
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        break;  
     
      case 15:
        I2C_SetEMail_login(i2c_buff[6*10+I2C_MSG_BUFF+1]);
        I2C_ClearMsgBuf();
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        break;   
  
      case 16:
        I2C_SetEMail_smtp(i2c_buff[6*10+I2C_MSG_BUFF+1]);
        I2C_ClearMsgBuf();
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        break;
  
      case 17:
        I2C_SetEMail_port(i2c_buff[6*10+I2C_MSG_BUFF+1]);
        I2C_ClearMsgBuf();
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        break;
  
      case 18:
        I2C_SetEMail_pwd(i2c_buff[6*10+I2C_MSG_BUFF+1]);
        I2C_ClearMsgBuf();
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        break;      
        
      case 19:
        I2C_Set_PinCode(i2c_buff[6*10+I2C_MSG_BUFF+1]);
        I2C_ClearMsgBuf();
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        break;   
        
      case 20:
        I2C_ClearMsgBuf();
        memcpy(i2c_buff+60, pincode, PIN_BUFFER) ;
        memcpy(i2c_buff+60+PIN_BUFFER, phone, PHONE_BUFFER) ;
        memcpy(i2c_buff+60+PIN_BUFFER+PHONE_BUFFER, email, EMAIL_BUFFER) ;
        memcpy(i2c_buff+60+PIN_BUFFER+PHONE_BUFFER+EMAIL_BUFFER, email_login, EMAIL_BUFFER) ;
        memcpy(i2c_buff+60+PIN_BUFFER+PHONE_BUFFER+EMAIL_BUFFER+EMAIL_BUFFER, email_smtp, SMTP_BUFFER) ;
        memcpy(i2c_buff+60+PIN_BUFFER+PHONE_BUFFER+EMAIL_BUFFER+EMAIL_BUFFER+SMTP_BUFFER, email_port, PORT_BUFFER) ;
        memcpy(i2c_buff+60+PIN_BUFFER+PHONE_BUFFER+EMAIL_BUFFER+EMAIL_BUFFER+SMTP_BUFFER+PORT_BUFFER, email_pwd, PWD_BUFFER) ;
  
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        i2c_buff[6*10+I2C_MSG_BUFF+2]=1;
        break;   
        
      //GPRS functions 
        
      case 21:
        I2C_Send_AT(i2c_buff[6*10+I2C_MSG_BUFF+1]);
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        i2cActiveTime=6000;
        break;   
        
      case 22:
        SIM900_call_to(phone);
        I2C_ClearMsgBuf();
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        break;    
  
      case 23:
        SIM900_send_sms(phone,s);
        I2C_ClearMsgBuf();
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        break;    
        
      case 24:
        SIM900_sendMail(email, "Message from SmartHome", s);
        I2C_ClearMsgBuf();
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        break;
        
      case 25:
        SIM900_get_sms_list();
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        i2c_buff[6*10+I2C_MSG_BUFF+2]=1;
        break;   
        
      case 26:
        SIM900_get_sms_info(i2c_buff[6*10+I2C_MSG_BUFF+1]);
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        i2c_buff[6*10+I2C_MSG_BUFF+2]=1;
        break;       
   
      case 27:
        SIM900_get_sms_body(i2c_buff[6*10+I2C_MSG_BUFF+1]);
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        i2c_buff[6*10+I2C_MSG_BUFF+2]=1;
        break;   
        
      case 28:
        SIM900_delete_sms(i2c_buff[6*10+I2C_MSG_BUFF+1]);
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        break;   
        
      case 29:
        SIM900_delete_all_sms();
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        break;   
        
      case 30:
        SIM900_get_score();
        i2c_buff[6*10+I2C_MSG_BUFF]=0;
        break;   
        
  
      default:
      break;
    }

    //for debug
    if(Serial.available())
    {
      char t=Serial.read();
      switch(t)
      {       
      case '9':
        SIM900.print(F("AT+CUSD=1,\"*121#\"\r"));
        break;
        
      case '8':
        SIM900.print(F("AT+CUSD=1,\"*111#\"\r"));
        break;
        
      case '0':
        Serial.println(pincode);
        Serial.println(phone);
        Serial.println(email);
        Serial.println(email_login);
        Serial.println(email_smtp);
        Serial.println(email_port);
        Serial.println(email_pwd);
        break;
        
      default:
        Serial.print(F("***"));
        for(int j=0;j<42;j++)
        {
          Serial.print(F("\r\n"));
          Serial.print(j);
          Serial.print(F(" - "));
          for(int i=0;i<6;i++)
          {
            Serial.print(F(" "));
            Serial.print(i2c_buff[i+j*6]);
          }
        }
        break;
      } 
    }
  }
  
  //GAS
  for(int x = 0 ; x < 6 ; x+=2)
  {
    if(i2c_buff[x]>0 && alerts[0]==0)
    {
      char msg[30]={},msg2[30]={};
      strcpy_P(msg, (char*)pgm_read_word(&(alert_names[0])));
      strcpy_P(msg2, (char*)pgm_read_word(&(alert_names[1])));
      
      if(SIM900_send_sms(phone,msg2))alerts[0]=1; 
      else delay(30000);
      
      if(SIM900_sendMail(email, msg, msg2))alerts[0]=1; 
      else delay(30000);
    }
  }
  
  //WATER
  for(int x = 0 ; x < 6 ; x+=2)
  {
    if(i2c_buff[x+6]>0 && alerts[1]==0)
    {
      char msg[30]={},msg2[30]={};
      strcpy_P(msg, (char*)pgm_read_word(&(alert_names[0])));
      strcpy_P(msg2, (char*)pgm_read_word(&(alert_names[2])));
          
      if(SIM900_send_sms(phone,msg2))alerts[1]=1; 
      else delay(30000);
      
      if(SIM900_sendMail(email, msg, msg2))alerts[1]=1; 
      else delay(30000);
    }
  }
  
  //ELECTRICITY
  for(int x = 0 ; x < 6 ; x+=2)
  {
    if(i2c_buff[x+12]<6 && alerts[2]==0)
    {
      char msg[30]={},msg2[30]={};
      strcpy_P(msg, (char*)pgm_read_word(&(alert_names[0])));
      strcpy_P(msg2, (char*)pgm_read_word(&(alert_names[2])));
  
      if(SIM900_send_sms(phone,msg2))alerts[2]=1; 
      else delay(30000);
      
      if(SIM900_sendMail(email, msg, msg2))alerts[2]=1; 
      else delay(30000);
    }
  }
  
  //ALARM
  for(int x = 0 ; x < 6 ; x+=2)
  {
    if(i2c_buff[x+18]>0 && alerts[3]==0)
    {
      char msg[30]={},msg2[30]={};
      strcpy_P(msg, (char*)pgm_read_word(&(alert_names[0])));
      strcpy_P(msg2, (char*)pgm_read_word(&(alert_names[2])));
            
      if(SIM900_send_sms(phone,msg2))alerts[3]=1; 
      else delay(30000);
      
      if(SIM900_sendMail(email, msg, msg2))alerts[3]=1; 
      else delay(30000);
    }
  }
  
  //INTERNET
  for(int x = 0 ; x < 6 ; x+=2)
  {
    if(i2c_buff[x+24]>2 && alerts[4]==0)
    {
      char msg[30]={},msg2[30]={};
      strcpy_P(msg, (char*)pgm_read_word(&(alert_names[0])));
      strcpy_P(msg2, (char*)pgm_read_word(&(alert_names[2])));
      
      if(SIM900_send_sms(phone,msg2))alerts[4]=1; 
      else delay(30000);
      
      if(SIM900_sendMail(email, msg, msg2))alerts[4]=1; 
      else delay(30000);
    }
  }
} 

