void SIM900_pin_config()
{
  pinMode(PIN_POWER, OUTPUT);
  pinMode(PIN_RESET, OUTPUT);
}

void  SIM900_power()
{
  digitalWrite(PIN_POWER,LOW);
  delay(1000);
  digitalWrite(PIN_POWER,HIGH);
  Serial.println(F("SIM900 power OFF")); 
  delay(2000);
  digitalWrite(PIN_POWER,LOW);
  delay(3000);
  Serial.println(F("SIM900 power ON")); 
}

void SIM900_reset()
{
  digitalWrite(PIN_RESET, HIGH);
  delay(30);
  digitalWrite(PIN_RESET, LOW);
  delay(3000);  
  Serial.println(F("SIM900 reset")); 
}

boolean SIM900_get_score()
{
  SIM900.print(F("AT+CUSD=1,\"*111#\"\r"));
  if(SIM900_wait("+CUSD:",MESSAGE_TIMEOUT+10,SYMBOL_TIMEOUT,uartbuf))
  {
    char floatbuf[10]={}; 
    char *p,*s;
    if((NULL!=(p=strstr(uartbuf,"s")))&&(NULL!=(s=strstr(uartbuf,"h"))))
    {
      memcpy(floatbuf, p+2, (s-p)) ;
      scoreHRN = atof(floatbuf); 
      scoreKOP = (atof(floatbuf)-scoreHRN)*100; 
      
      //Serial.println(floatbuf);  
      Serial.print(F("Score: ")); 
      Serial.print(scoreHRN); 
      Serial.print("."); 
      Serial.println(scoreKOP); 
      return true;
    }
    else return false;
  }
  else return false;
}

boolean SIM900_setup()
{
  SIM900.print(F("ATE0\r"));
  if(!SIM900_waitOK())Serial.println(F("Error: Echo config failed"));
  
  SIM900.print(F("AT+DDET=1,5,300,1,100,224,0\r"));
  if(!SIM900_waitOK())Serial.println(F("Error: DTMF config failed"));

  SIM900.print(F("AT+CMGF=1\r"));
  if(!SIM900_waitOK())Serial.println(F("Error: Text mod config failed"));
  
  SIM900.print(F("AT+CSCS=\"GSM\"\r"));
  if(!SIM900_waitOK())Serial.println(F("Error: Text gsm config failed"));

  return SIM900_get_score();
}

boolean SIM900_answer()
{
  SIM900.print(F("ATA\r"));
  return SIM900_waitOK();
}

boolean SIM900_hangup()
{
  SIM900.print(F("ATH\r"));
  return SIM900_waitOK();
}

boolean SIM900_listen()
{
  if(SIM900.available()) 
  {
    memset(uartbuf,0,UART_BUFFER);
    return SIM900_wait("",NO_TIMEOUT,SYMBOL_TIMEOUT,uartbuf);
  }
  else delay(1);

  return false;
}

boolean SIM900_wait(char* resp, unsigned int timeout, unsigned int stimeout, char* retval)
{
  int len = strlen(resp), sum = 0;

  unsigned long timerStart = millis(),timeEnd;
  while(len>0 && timeout>0) 
  {
    if(SIM900.available()) 
    {
      char c = SIM900.read();
      //Serial.print(c);
      sum = (c==resp[sum]) ? sum+1 : 0;
      if(sum == len)break;
    }
    if ((unsigned long) (millis() - timerStart) > timeout * 1000UL)
    {
      return false;
    }
  }
      
  len=0;
  timeEnd=millis()+stimeout;
  while(millis()<timeEnd)
  {
    if(SIM900.available()) 
    {
      retval[len] = SIM900.read();
      //Serial.print(retval[len]);
      timeEnd=millis()+stimeout;
      len++;
    }
  }
  return true;
}

boolean SIM900_powerOFF()
{
  SIM900.print(F("AT+CPOWD=1\r"));
  return SIM900_wait("NORMAL POWER DOWN",MESSAGE_TIMEOUT,NO_TIMEOUT,uartbuf);
}

boolean SIM900_waitOK()
{
  return SIM900_wait("OK\r\n",MESSAGE_TIMEOUT,NO_TIMEOUT,uartbuf);
}

boolean SIM900_disconnect_gprs()
{
  SIM900.print(F("AT+SAPBR=0,1\r"));
  if(!SIM900_waitOK())
  {
    Serial.println(F("Error: GPRS close connection failed"));
    return false;
  }
  delay(1000);
  return true;
}

boolean SIM900_check_gprs_attach()
{
  SIM900.print(F("AT+CGATT?\r"));
  if(SIM900_wait("+CGATT:",MESSAGE_TIMEOUT,SYMBOL_TIMEOUT,uartbuf))
  {
    char *p;
    p = strstr(uartbuf," ");
    if((char)*(p+1)=='1')return true;
  }
  return false;
}

boolean SIM900_check_connection_gprs()
{
  SIM900.print(F("AT+SAPBR=2,1\r"));
  char gprsBuffer[48]; 
  char *p,*p2;

  if(SIM900_wait("+SAPBR:",MESSAGE_TIMEOUT,SYMBOL_TIMEOUT,uartbuf))
  {   
    p = strstr(uartbuf,",");
    if((char)*(p+1)=='1')
    {
      Serial.print(F("Connection established, your IP is "));
      p2=p+4;
      while((char)*(p2)!='\"')
      {
        Serial.print((char)*(p2));
        p2++;	
      }
      Serial.print(F("\r\n"));
      return true;
    }
    else if((char)*(p+1)=='0')
    {
      Serial.println(F("Connection establishing, wait"));
      return false;
    }
    else if((char)*(p+1)=='2')
    {
      Serial.println(F("Connection closing, wait"));
      return true;
    }
    else if((char)*(p+1)=='3')
    {
      Serial.println(F("Connection closed"));
      return false;
    }
  }
  return SIM900_waitOK();
}

boolean SIM900_connect_gprs()
{
  if(!SIM900_check_gprs_attach())
  {
    Serial.println(F("Attach GPRS failed"));
    return false;
  }

  SIM900.print(F("AT+SAPBR=3,1,\"CONTYPE\",\"GPRS\"\r"));
  if(!SIM900_waitOK())
  {
    Serial.println(F("Configure bearer profile 1 failed"));
    return false;
  }
  delay(500);

  SIM900.print(F("AT+SAPBR=3,1,\"APN\",\"internet\"\r"));
  if(!SIM900_waitOK())
  {
    Serial.println(F("Configure bearer profile 2 failed"));
    return false;
  }
  delay(500);

  SIM900.print(F("AT+SAPBR=3,1,\"USER\",\"\"\r"));
  if(!SIM900_waitOK())
  {
    Serial.println(F("Configure bearer profile 3 failed"));
    return false;
  }
  delay(500);

  SIM900.print(F("AT+SAPBR=3,1,\"PWD\",\"\"\r"));
  if(!SIM900_waitOK())
  {
    Serial.println(F("Configure bearer profile 4 failed"));
    return false;
  }
  delay(500);

  SIM900.print(F("AT+SAPBR=1,1\r"));
  if(!SIM900_wait("OK\r\n",MESSAGE_TIMEOUT+5,NO_TIMEOUT,uartbuf))
  {
    Serial.println(F("Open a GPRS context failed"));
    return false;
  }

  SIM900_check_connection_gprs();

  return true;
}

boolean SIM900_set_gprs_mail_param()
{
  SIM900.print(F("AT+EMAILCID=1\r"));
  if(!SIM900_waitOK())
  {
    Serial.println(F("Error: Set parameters of Email CID failed"));
    return false;
  }

  SIM900.print(F("AT+EMAILTO=30\r"));
  if(!SIM900_waitOK())
  {
    Serial.println(F("Error: Set parameters of Email TO failed"));
    return false;
  }

  SIM900.print(F("AT+EMAILSSL=1\r"));
  if(!SIM900_waitOK())
  {
    Serial.println(F("Error: Set parameters of Email SSL failed"));
    return false;
  }

  return true;
}

boolean SIM900_set_smtp_server(char *server, char *port)
{
  SIM900.print(F("AT+SMTPSRV=\""));
  SIM900.print(server);
  SIM900.print(F("\","));
  SIM900.print(port);
  SIM900.print(F("\r"));
  return SIM900_waitOK();
}

boolean SIM900_set_smtp_auth(char *login, char *pass)
{
  SIM900.print(F("AT+SMTPAUTH=1,\""));
  SIM900.print(login);
  SIM900.print(F("\",\""));
  SIM900.print(pass);
  SIM900.print(F("\"\r"));
  return SIM900_waitOK();
}

boolean SIM900_set_smtp_from(char *mail, char *user)
{
  SIM900.print(F("AT+SMTPFROM=\""));
  SIM900.print(mail);
  SIM900.print(F("\",\""));
  SIM900.print(user);
  SIM900.print(F("\"\r"));
  return SIM900_waitOK();
}

boolean SIM900_set_smtp_recp(char *mail)
{
  SIM900.print(F("AT+SMTPRCPT=0,0,\""));
  SIM900.print(mail);
  SIM900.print(F("\"\r"));
  return SIM900_waitOK();
}

boolean SIM900_set_smtp_sub(char *sub)
{
  SIM900.print(F("AT+SMTPSUB=\""));
  SIM900.print(sub);
  SIM900.print(F("\"\r"));
  return SIM900_waitOK();
}

boolean SIM900_send_gprs_mail()
{
  SIM900.print(F("AT+SMTPSEND\r"));
  return SIM900_waitOK();
}

boolean SIM900_set_mail_body(char *data)
{
  SIM900.print(F("AT+SMTPBODY\r"));
  if(!SIM900_wait(">",MESSAGE_TIMEOUT,NO_TIMEOUT,uartbuf))
  {
    Serial.println(F("Error: Set mail body failed"));
    return false;
  }
  SIM900.print(data);

  delay(500);
  SIM900.print(EndMark);
  return SIM900_waitOK();
}

boolean SIM900_change_mic_channel(char *ch)
{
  SIM900.print(F("AT+CHFA="));
  SIM900.print(ch);
  SIM900.print(F("\r"));
  return SIM900_waitOK();
}

boolean SIM900_change_mic_gain(char *ch,char *gain)
{
  SIM900.print(F("AT+CMIC="));
  SIM900.print(ch);
  SIM900.print(F(","));
  SIM900.print(gain);
  SIM900.print(F("\r"));
  return SIM900_waitOK();
}

boolean SIM900_amr_play(char *amr)
{
  SIM900.print(F("AT+CPAMR=\""));
  SIM900.print(amr);
  SIM900.print(F("\",0\r"));
  return SIM900_waitOK();
}

boolean SIM900_amr_stop()
{
  SIM900.print(F("AT+CPAMR=0\r"));
  return SIM900_waitOK();
}

/*
The format of command should be: AT+DDET=mode,discard,delay,show,cyc,para,showabcd

Mode: 0 or 1(default 0)-------enable DTMF decoder or not, 1 for enable; 0 for disable
Discard--------- discard the tone time in mm unit for detecting. Default 5.
Delay--------The delay time in mm unit for opening detecting after incoming call,default 300
show----------0 or 1,default 0---------show the circule of detected DTMF tone or not
cyc-----------100 or 200, default 200------scanning time
para---------244. reserved by SIMCom
showabcd--- 0 or 1, default 0---------detect the tone of A,B,C, D or not. 
*/

boolean SIM900_dtmf_enable(char *dtmf)
{
  SIM900.print(F("AT+DDET="));
  SIM900.print(dtmf);
  SIM900.print(F("\r"));
  return SIM900_waitOK();
}

boolean SIM900_sendMail(char *mail, char *subject, char *body)
{
  delay(2000);
  if(SIM900_check_connection_gprs())SIM900_disconnect_gprs();
  if(!SIM900_connect_gprs())return false;
  if(!SIM900_set_gprs_mail_param())return false;
  if(!SIM900_set_smtp_server(email_smtp, email_port))return false;
  if(!SIM900_set_smtp_auth(email_login, email_pwd))return false;
  if(!SIM900_set_smtp_from(email_login, "SmartHome"))return false;
  if(!SIM900_set_smtp_recp(mail))return false;
  if(!SIM900_set_smtp_sub(subject))return false;
  if(!SIM900_set_mail_body(body))return false;
  if(!SIM900_send_gprs_mail())return false;
  return true;
}

boolean SIM900_send_sms(char *number,char *data)
{
  SIM900.print(F("AT+CMGF=1\r"));
  if(!SIM900_waitOK())
  {
    Serial.println(F("Error: Set SMS to text mode failed"));
    return false;
  }
  
  SIM900.print(F("AT+CMGS=\""));
  SIM900.print(number);
  SIM900.print(F("\"\r"));
  
  if(!SIM900_wait(">",MESSAGE_TIMEOUT+5,NO_TIMEOUT,uartbuf))
  {
    Serial.println(F("Error: Set sms body failed"));
    return false;
  }
  SIM900.print(data);
  SIM900.print(EndMark);
  return SIM900_wait("OK\r\n",MESSAGE_TIMEOUT+5,SYMBOL_TIMEOUT,uartbuf);
}

boolean SIM900_call_to(char *number)
{
  i2c_buff[6*10+I2C_MSG_BUFF+2]=3;
  pinOk=0;
  pinCount=0;
  ringActive=1;
  memset(pinbuf,0,PIN_BUFFER);
  SIM900_change_mic_gain("0","0");
  SIM900.print(F("ATD"));
  SIM900.print(number);
  SIM900.print(F(";\r"));
  return SIM900_wait("OK\r\n",MESSAGE_TIMEOUT+2,SYMBOL_TIMEOUT,uartbuf);
}

boolean SIM900_get_sms_list()
{
  unsigned long timerStart = millis(),timeEnd;
  SIM900.print(F("AT+CMGL=\"ALL\"\r"));
  I2C_ClearMsgBuf();
  memset(uartbuf,0,UART_BUFFER);
  char *s,*p;
  int i2c_count=0;
  while(NULL == strstr(uartbuf,"OK\r\n"))
  {
    while(SIM900.available()) 
    {
      char c = SIM900.read();
      //Serial.print(c);
      uartbuf[10]=c;      
      for(int i=1;i<11;i++) uartbuf[i-1] = uartbuf[i];
      if(uartbuf[0]=='+' && uartbuf[1]=='C' && uartbuf[2]=='M' && uartbuf[3]=='G' && uartbuf[4]=='L' && uartbuf[5]==':'  )
      {
        if(i2c_count<I2C_MSG_BUFF)
        {
          if(NULL!=(p=strstr(uartbuf,",")))
          {
            memcpy(i2c_buff+60+i2c_count, uartbuf+7, (p-6-uartbuf));
            i2c_count+=(p-6-uartbuf);
          }
        }
      }
      timerStart = millis();
    }
    
    if ((unsigned long) (millis() - timerStart) > 2 * 1000UL)return false;
  }
  return true;
}


boolean SIM900_delete_sms(int num)
{
  if(num>4)return 0;
  char sms[4]={};
  memcpy(sms, i2c_buff+60, num) ;
  
  SIM900.print(F("AT+CMGD="));
  SIM900.print(sms);
  SIM900.print(F("\r"));
  
  I2C_ClearMsgBuf();
  
  return SIM900_waitOK();
}

boolean SIM900_delete_all_sms()
{
  SIM900.print(F("AT+CMGDA=\"DEL ALL\"\r"));
  return SIM900_waitOK();
}

boolean SIM900_get_sms_info(int num)
{
  if(num>4)return 0;
  char sms[4]={};
  memcpy(sms, i2c_buff+60, num) ;
  
  SIM900.print(F("AT+CMGR="));
  SIM900.print(sms);
  SIM900.print(F("\r"));
  
  I2C_ClearMsgBuf();
  
  if(SIM900_wait("+CMGR: ",MESSAGE_TIMEOUT+2,SYMBOL_TIMEOUT,uartbuf))
  {
    char *s;
    s=strstr(uartbuf,"\r\n");
    if((s-uartbuf)<I2C_MSG_BUFF)memcpy(i2c_buff+60, uartbuf, (s-uartbuf)) ;
  }
}

boolean SIM900_get_sms_body(int num)
{
  if(num>4)return 0;
  char sms[4]={};
  memcpy(sms, i2c_buff+60, num) ;
  
  SIM900.print(F("AT+CMGR="));
  SIM900.print(sms);
  SIM900.print(F("\r"));
  
  I2C_ClearMsgBuf();
  
  if(SIM900_wait("+CMGR: ",MESSAGE_TIMEOUT+2,SYMBOL_TIMEOUT,uartbuf))
  {
    char *s,*p,*t;
    if(NULL!=(s=strstr(uartbuf,"\r\n")))
    {
      s=s+2;
      if(NULL!=(p=strstr(s,"\r\n")))
      {
        *p=0;
        if((p-s)<160)
        {
          t=s;
          char tmp;
          int ctmp=0;
          while((tmp = *t++))
          {
             if((tmp>='0' && tmp<='9')||(tmp>='A' && tmp<='F'))ctmp++;
             else 
             {
               ctmp=0;
               break;               
             }
          }
          if(ctmp==0)memcpy(i2c_buff+60, s, (p-s));
          else translate(s,i2c_buff+60);
        }
        else if(((p-s)/2)<I2C_MSG_BUFF)translate(s,i2c_buff+60);
      }
    }
  }
}

const char S0[] PROGMEM = "A";  
const char S1[] PROGMEM = "B";
const char S2[] PROGMEM = "V";
const char S3[] PROGMEM = "G";
const char S4[] PROGMEM = "D";
const char S5[] PROGMEM = "E";
const char S6[] PROGMEM = "ZH";
const char S7[] PROGMEM = "Z";
const char S8[] PROGMEM = "I";
const char S9[] PROGMEM = "Y";

const char S10[] PROGMEM = "K";  
const char S11[] PROGMEM = "L";
const char S12[] PROGMEM = "M";
const char S13[] PROGMEM = "N";
const char S14[] PROGMEM = "O";
const char S15[] PROGMEM = "P";
const char S16[] PROGMEM = "R";
const char S17[] PROGMEM = "S";
const char S18[] PROGMEM = "T";
const char S19[] PROGMEM = "U";

const char S20[] PROGMEM = "F";  
const char S21[] PROGMEM = "H";
const char S22[] PROGMEM = "C";
const char S23[] PROGMEM = "CH";
const char S24[] PROGMEM = "SH";
const char S25[] PROGMEM = "SH";
const char S26[] PROGMEM = "'";
const char S27[] PROGMEM = "'";
const char S28[] PROGMEM = "'";
const char S29[] PROGMEM = "E";

const char S38[] PROGMEM = "YU";
const char S39[] PROGMEM = "YA";

const char S40[] PROGMEM = "a";  
const char S41[] PROGMEM = "b";
const char S42[] PROGMEM = "v";
const char S43[] PROGMEM = "g";
const char S44[] PROGMEM = "d";
const char S45[] PROGMEM = "e";
const char S46[] PROGMEM = "zh";
const char S47[] PROGMEM = "z";
const char S48[] PROGMEM = "i";
const char S49[] PROGMEM = "y";

const char S50[] PROGMEM = "k";  
const char S51[] PROGMEM = "l";
const char S52[] PROGMEM = "m";
const char S53[] PROGMEM = "n";
const char S54[] PROGMEM = "o";
const char S55[] PROGMEM = "p";
const char S56[] PROGMEM = "r";
const char S57[] PROGMEM = "s";
const char S58[] PROGMEM = "t";
const char S59[] PROGMEM = "u";

const char S60[] PROGMEM = "f";  
const char S61[] PROGMEM = "h";
const char S62[] PROGMEM = "c";
const char S63[] PROGMEM = "ch";
const char S64[] PROGMEM = "sh";
const char S65[] PROGMEM = "sh";
const char S66[] PROGMEM = "'";
const char S67[] PROGMEM = "'";
const char S68[] PROGMEM = "'";
const char S69[] PROGMEM = "e";

const char S78[] PROGMEM = "yu";
const char S79[] PROGMEM = "ya";

const char* const char_table[64] PROGMEM = 
{
S0, S1, S2, S3, S4, S5, S6, S7, S8, S9,
S10, S11, S12, S13, S14, S15, S16, S17, S18, S19,
S20, S21, S22, S23, S24, S25, S26, S27, S28, S29,
S38, S39,
S40, S41, S42, S43, S44, S45, S46, S47, S48, S49,
S50, S51, S52, S53, S54, S55, S56, S57, S58, S59,
S60, S61, S62, S63, S64, S65, S66, S67, S68, S69,
S78, S79
};

inline unsigned char toHex( char ch )
{
   return ( ( ch >= 'A' ) ? ( ch - 'A' + 0xA ) : ( ch - '0' ) ) & 0x0F;
}

/*
int convert( char* str, char* bytes ) 
{
   unsigned char Hi, Lo;

   int i = 0;
   while( ( Hi = *str++ ) && ( Lo = *str++ ) )
   {
      bytes[i++] = ( toHex( Hi ) << 4 ) | toHex( Lo ); 
   }
   
   return i;
}
*/

boolean translate(char* str, char* buf) 
{
   unsigned char Hi, Lo, tbyte,sbyte;
  
   while((Hi = *str++)&&(Lo = *str++))
   {
      tbyte = (toHex(Hi) << 4) | toHex(Lo); 
      Hi = *str++;
      Lo = *str++;
      sbyte=((toHex(Hi) << 4) | toHex(Lo));
      
      if(tbyte==0)
      {
        int index = strlen(buf);
        buf[index++] = sbyte;
        buf[index] = '\0';
      }
      else
      {
        uint16_t symbol=tbyte;
        symbol=(symbol<<8)+sbyte;
        if(symbol>=0x0410 && symbol<=0x044F)strcat_P(buf, (char*)pgm_read_word(&(char_table[symbol-0x0410])));
      }
   }
  return true;
}
